package uci.inf221.lunarLander.arch.c2;

import org.apache.log4j.Logger;

import uci.inf221.lunarLander.arch.Component;
import uci.inf221.lunarLander.arch.Port;

public abstract class C2Component extends Component {

	private final Logger log = Logger.getLogger(this.getClass());

	public C2Component() {
		super();

		createPorts();
		initialize();
	}

	protected abstract void createPorts();

	protected abstract void initialize();

	public abstract void start();

	public abstract void stop();

	public abstract void waitForStop();

	protected abstract void receiveNotification(C2Notification notification);

	protected abstract void receiveRequest(C2Request request);

	protected C2TopPort addTopPort(String identifier) {
		C2TopPort port = new TopPort();
		addPort(identifier, port);
		return port;
	}

	protected C2BottomPort addBottomPort(String identifier) {
		C2BottomPort port = new BottomPort();
		addPort(identifier, port);
		return port;
	}

	public void sendNotification(C2Notification message) {
		log.trace("sending notification: " + message.getClass().getSimpleName());
		for (Port port : ports.values()) {
			if (port instanceof BottomPort) {
				((BottomPort) port).sendNotification(message);
			}
		}
	}

	public void sendRequest(C2Request message) {
		log.trace("sending request: " + message.getClass().getSimpleName());
		for (Port port : ports.values()) {
			if (port instanceof TopPort) {
				((TopPort) port).sendRequest(message);
			}
		}
	}

	public class TopPort extends C2TopPort {
		@Override
		protected void receiveNotification(C2Notification notification) {
			log.trace("receiving notification: " + notification.getClass().getSimpleName());
			C2Component.this.receiveNotification(notification);
		}
	}

	public class BottomPort extends C2BottomPort {
		@Override
		protected void receiveRequest(C2Request request) {
			log.trace("receiving request: " + request.getClass().getSimpleName());
			C2Component.this.receiveRequest(request);
		}
	}
}
