package uci.inf221.lunarLander.dataStore.component;

import java.util.Map;
import java.util.UUID;
import java.util.concurrent.ConcurrentHashMap;

import uci.inf221.lunarLander.arch.c2.C2Component;
import uci.inf221.lunarLander.arch.c2.C2Notification;
import uci.inf221.lunarLander.arch.c2.C2Request;
import uci.inf221.lunarLander.dataStore.entities.Asteroid;
import uci.inf221.lunarLander.dataStore.entities.DataEntity;
import uci.inf221.lunarLander.dataStore.entities.MovementVector;
import uci.inf221.lunarLander.dataStore.message.CreateNewAsteroidRequest;
import uci.inf221.lunarLander.dataStore.message.CurrentEntityDataNotification;
import uci.inf221.lunarLander.dataStore.message.DestroyAllEntitiesRequest;
import uci.inf221.lunarLander.dataStore.message.DestroyEntityRequest;
import uci.inf221.lunarLander.dataStore.message.EntityDestroyedNotification;
import uci.inf221.lunarLander.dataStore.message.GetCurrentDataRequest;
import uci.inf221.lunarLander.dataStore.message.MovementVectorUpdatedNotification;
import uci.inf221.lunarLander.dataStore.message.NewAsteroidNotification;
import uci.inf221.lunarLander.dataStore.message.UpdateMovementVectorAccelerationRequest;
import uci.inf221.lunarLander.dataStore.message.UpdateMovementVectorPositionAndSpeedRequest;
import uci.inf221.lunarLander.dataStore.message.UpdateMovementVectorRequest;

public class AsteroidsComponent extends C2Component {

	public static final String PORT_IDENTIFIER_DATA_STORE_CONNECTOR = "data_store";

	private Map<UUID, Asteroid> asteroids = new ConcurrentHashMap<UUID, Asteroid>();

	@Override
	protected void createPorts() {
		addBottomPort(PORT_IDENTIFIER_DATA_STORE_CONNECTOR);
	}

	@Override
	protected void initialize() {
	}

	@Override
	public void start() {
	}

	@Override
	public void stop() {
	}

	@Override
	public void waitForStop() {
	}

	@Override
	protected void receiveNotification(C2Notification notification) {
	}

	@Override
	protected void receiveRequest(C2Request request) {
		if (request instanceof CreateNewAsteroidRequest) {
			Asteroid asteroid = ((CreateNewAsteroidRequest) request)
					.getAsteroid();
			asteroids.put(asteroid.getId(), asteroid);
			NewAsteroidNotification notif = new NewAsteroidNotification(this,
					asteroid);
			notif.setInitiatorRequest(request);
			sendNotification(notif);
		} else if (request instanceof UpdateMovementVectorAccelerationRequest) {
			UUID entityId = ((UpdateMovementVectorAccelerationRequest) request)
					.getEntityId();
			Asteroid asteroid = asteroids.get(entityId);
			if (asteroid != null) {
				float[] acceleration = ((UpdateMovementVectorAccelerationRequest) request)
						.getAcceleration();
				MovementVector movementVector = asteroid.getMovementVector();
				synchronized (movementVector) {
					movementVector.setAcceleration(acceleration);
				}
				MovementVectorUpdatedNotification notif = new MovementVectorUpdatedNotification(
						this, entityId, movementVector);
				notif.setInitiatorRequest(request);
				sendNotification(notif);
			}
		} else if (request instanceof UpdateMovementVectorPositionAndSpeedRequest) {
			UUID entityId = ((UpdateMovementVectorPositionAndSpeedRequest) request)
					.getEntityId();
			Asteroid asteroid = asteroids.get(entityId);
			if (asteroid != null) {
				float[] position = ((UpdateMovementVectorPositionAndSpeedRequest) request)
						.getPosition();
				float[] speed = ((UpdateMovementVectorPositionAndSpeedRequest) request)
						.getSpeed();
				long clockTick = ((UpdateMovementVectorPositionAndSpeedRequest) request)
						.getClockTick();
				MovementVector movementVector = asteroid.getMovementVector();
				synchronized (movementVector) {
					movementVector.setPosition(position);
					movementVector.setSpeed(speed);
					movementVector.setClockTick(clockTick);
				}
				MovementVectorUpdatedNotification notif = new MovementVectorUpdatedNotification(
						this, entityId, movementVector);
				notif.setInitiatorRequest(request);
				sendNotification(notif);
			}
		} else if (request instanceof UpdateMovementVectorRequest) {
			UUID entityId = ((UpdateMovementVectorRequest) request)
					.getEntityId();
			Asteroid asteroid = asteroids.get(entityId);
			if (asteroid != null) {
				MovementVector newVector = ((UpdateMovementVectorRequest) request)
						.getMovementVector();
				MovementVector movementVector = asteroid.getMovementVector();
				synchronized (movementVector) {
					movementVector.setPosition(newVector.getPosition());
					movementVector.setSpeed(newVector.getSpeed());
					movementVector.setAcceleration(newVector.getAcceleration());
					movementVector.setClockTick(newVector.getClockTick());
				}
				MovementVectorUpdatedNotification notif = new MovementVectorUpdatedNotification(
						this, entityId, movementVector);
				notif.setInitiatorRequest(request);
				sendNotification(notif);
			}
		} else if (request instanceof DestroyEntityRequest) {
			UUID entityId = ((DestroyEntityRequest) request).getEntityId();
			if (asteroids.remove(entityId) != null) {
				EntityDestroyedNotification notif = new EntityDestroyedNotification(
						this, entityId);
				notif.setInitiatorRequest(request);
				sendNotification(notif);
			}
		} else if (request instanceof GetCurrentDataRequest) {
			CurrentEntityDataNotification notif = new CurrentEntityDataNotification(
					this, asteroids.values().toArray(
							new DataEntity[asteroids.size()]));
			notif.setInitiatorRequest(request);
			sendNotification(notif);
		} else if (request instanceof DestroyAllEntitiesRequest) {
			asteroids.clear();
		}
	}

}
