package uci.inf221.lunarLander.dataStore.component;

import uci.inf221.lunarLander.arch.c2.C2Component;
import uci.inf221.lunarLander.arch.c2.C2Notification;
import uci.inf221.lunarLander.arch.c2.C2Request;
import uci.inf221.lunarLander.dataStore.entities.Level;
import uci.inf221.lunarLander.dataStore.entities.Level.LevelStatus;
import uci.inf221.lunarLander.dataStore.message.CurrentEntityDataNotification;
import uci.inf221.lunarLander.dataStore.message.DestroyAllEntitiesRequest;
import uci.inf221.lunarLander.dataStore.message.GetCurrentDataRequest;
import uci.inf221.lunarLander.dataStore.message.LevelStatusUpdatedNotification;
import uci.inf221.lunarLander.dataStore.message.LevelUpdatedNotification;
import uci.inf221.lunarLander.dataStore.message.StartGameRequest;
import uci.inf221.lunarLander.dataStore.message.UpdateLevelMapRequest;
import uci.inf221.lunarLander.dataStore.message.UpdateLevelStatusRequest;

public class LevelComponent extends C2Component {

	public static final String PORT_IDENTIFIER_DATA_STORE_CONNECTOR = "data_store";

	private Level level;

	@Override
	protected void createPorts() {
		addBottomPort(PORT_IDENTIFIER_DATA_STORE_CONNECTOR);
	}

	@Override
	protected void initialize() {
		level = new Level();
		level.setLevelStatus(LevelStatus.INITIALISING);
	}

	@Override
	public void start() {
	}

	@Override
	public void stop() {
	}

	@Override
	public void waitForStop() {
	}

	@Override
	protected void receiveNotification(C2Notification notification) {
	}

	@Override
	protected void receiveRequest(C2Request request) {
		if (request instanceof UpdateLevelStatusRequest) {
			LevelStatusUpdatedNotification notif = null;
			synchronized (level) {
				LevelStatus newLevelStatus = ((UpdateLevelStatusRequest) request)
						.getLevelStatus();
				if (level.getLevelStatus() != newLevelStatus) {
					level.setLevelStatus(newLevelStatus);
					notif = new LevelStatusUpdatedNotification(this,
							level.getLevelStatus());
				}
			}
			if (notif != null) {
				notif.setInitiatorRequest(request);
				sendNotification(notif);
			}
		} else if (request instanceof UpdateLevelMapRequest) {
			LevelUpdatedNotification notif;
			LevelStatusUpdatedNotification notif2 = null;

			synchronized (level) {
				level.setTextRepresentation(((UpdateLevelMapRequest) request)
						.getTextRepresentation());
				if (level.getLevelStatus() != LevelStatus.LOADED) {
					level.setLevelStatus(LevelStatus.LOADED);
					notif2 = new LevelStatusUpdatedNotification(this,
							LevelStatus.LOADED);
				}
				notif = new LevelUpdatedNotification(this, level);
			}

			if (notif2 != null) {
				notif2.setInitiatorRequest(request);
				sendNotification(notif2);
			}

			notif.setInitiatorRequest(request);
			sendNotification(notif);
		} else if (request instanceof GetCurrentDataRequest) {
			CurrentEntityDataNotification notif;
			synchronized (level) {
				notif = new CurrentEntityDataNotification(this, level);
			}
			notif.setInitiatorRequest(request);
			sendNotification(notif);
		} else if (request instanceof StartGameRequest) {
			LevelStatusUpdatedNotification notif = null;
			synchronized (level) {
				LevelStatus newLevelStatus = LevelStatus.STARTED;
				if (level.getLevelStatus() != newLevelStatus) {
					level.setLevelStatus(newLevelStatus);
					notif = new LevelStatusUpdatedNotification(this,
							level.getLevelStatus());
				}
			}
			if (notif != null) {
				notif.setInitiatorRequest(request);
				sendNotification(notif);
			}
		} else if (request instanceof DestroyAllEntitiesRequest) {
			// TODO handle?
		}
	}

}
