package uci.inf221.lunarLander.dataStore.component;

import java.util.Map;
import java.util.UUID;
import java.util.concurrent.ConcurrentHashMap;

import uci.inf221.lunarLander.arch.c2.C2Component;
import uci.inf221.lunarLander.arch.c2.C2Notification;
import uci.inf221.lunarLander.arch.c2.C2Request;
import uci.inf221.lunarLander.dataStore.entities.DataEntity;
import uci.inf221.lunarLander.dataStore.entities.MovementVector;
import uci.inf221.lunarLander.dataStore.entities.Ship;
import uci.inf221.lunarLander.dataStore.message.CreateNewShipRequest;
import uci.inf221.lunarLander.dataStore.message.CurrentEntityDataNotification;
import uci.inf221.lunarLander.dataStore.message.DestroyAllEntitiesRequest;
import uci.inf221.lunarLander.dataStore.message.DestroyEntityRequest;
import uci.inf221.lunarLander.dataStore.message.EntityDestroyedNotification;
import uci.inf221.lunarLander.dataStore.message.GetCurrentDataRequest;
import uci.inf221.lunarLander.dataStore.message.MovementVectorUpdatedNotification;
import uci.inf221.lunarLander.dataStore.message.NewShipNotification;
import uci.inf221.lunarLander.dataStore.message.UpdateMovementVectorAccelerationRequest;
import uci.inf221.lunarLander.dataStore.message.UpdateMovementVectorPositionAndSpeedRequest;
import uci.inf221.lunarLander.dataStore.message.UpdateMovementVectorRequest;

public class ShipsComponent extends C2Component {

	public static final String PORT_IDENTIFIER_DATA_STORE_CONNECTOR = "data_store";

	private Map<UUID, Ship> ships = new ConcurrentHashMap<UUID, Ship>();

	@Override
	protected void createPorts() {
		addBottomPort(PORT_IDENTIFIER_DATA_STORE_CONNECTOR);
	}

	@Override
	protected void initialize() {
	}

	@Override
	public void start() {
	}

	@Override
	public void stop() {
	}

	@Override
	public void waitForStop() {
	}

	@Override
	protected void receiveNotification(C2Notification notification) {
	}

	@Override
	protected void receiveRequest(C2Request request) {
		if (request instanceof CreateNewShipRequest) {
			Ship ship = ((CreateNewShipRequest) request).getShip();
			ships.put(ship.getId(), ship);
			NewShipNotification notif = new NewShipNotification(this, ship);
			notif.setInitiatorRequest(request);
			sendNotification(notif);
		} else if (request instanceof UpdateMovementVectorAccelerationRequest) {
			UUID entityId = ((UpdateMovementVectorAccelerationRequest) request)
					.getEntityId();
			Ship ship = ships.get(entityId);
			if (ship != null) {
				float[] acceleration = ((UpdateMovementVectorAccelerationRequest) request)
						.getAcceleration();
				MovementVector movementVector = ship.getMovementVector();
				synchronized (movementVector) {
					movementVector.setAcceleration(acceleration);
				}
				MovementVectorUpdatedNotification notif = new MovementVectorUpdatedNotification(
						this, entityId, movementVector);
				notif.setInitiatorRequest(request);
				sendNotification(notif);
			}
		} else if (request instanceof UpdateMovementVectorPositionAndSpeedRequest) {
			UUID entityId = ((UpdateMovementVectorPositionAndSpeedRequest) request)
					.getEntityId();
			Ship ship = ships.get(entityId);
			if (ship != null) {
				float[] position = ((UpdateMovementVectorPositionAndSpeedRequest) request)
						.getPosition();
				float[] speed = ((UpdateMovementVectorPositionAndSpeedRequest) request)
						.getSpeed();
				long clockTick = ((UpdateMovementVectorPositionAndSpeedRequest) request)
						.getClockTick();
				MovementVector movementVector = ship.getMovementVector();
				synchronized (movementVector) {
					movementVector.setPosition(position);
					movementVector.setSpeed(speed);
					movementVector.setClockTick(clockTick);
				}
				MovementVectorUpdatedNotification notif = new MovementVectorUpdatedNotification(
						this, entityId, movementVector);
				notif.setInitiatorRequest(request);
				sendNotification(notif);
			}
		} else if (request instanceof UpdateMovementVectorRequest) {
			UUID entityId = ((UpdateMovementVectorRequest) request)
					.getEntityId();
			Ship ship = ships.get(entityId);
			if (ship != null) {
				MovementVector newVector = ((UpdateMovementVectorRequest) request)
						.getMovementVector();
				MovementVector movementVector = ship.getMovementVector();
				synchronized (movementVector) {
					movementVector.setPosition(newVector.getPosition());
					movementVector.setSpeed(newVector.getSpeed());
					movementVector.setAcceleration(newVector.getAcceleration());
					movementVector.setClockTick(newVector.getClockTick());
				}
				MovementVectorUpdatedNotification notif = new MovementVectorUpdatedNotification(
						this, entityId, movementVector);
				notif.setInitiatorRequest(request);
				sendNotification(notif);
			}
		} else if (request instanceof DestroyEntityRequest) {
			UUID entityId = ((DestroyEntityRequest) request).getEntityId();
			if (ships.remove(entityId) != null) {
				EntityDestroyedNotification notif = new EntityDestroyedNotification(
						this, entityId);
				notif.setInitiatorRequest(request);
				sendNotification(notif);
			}
		} else if (request instanceof GetCurrentDataRequest) {
			CurrentEntityDataNotification notif = new CurrentEntityDataNotification(
					this, ships.values().toArray(new DataEntity[ships.size()]));
			notif.setInitiatorRequest(request);
			sendNotification(notif);
		} else if (request instanceof DestroyAllEntitiesRequest) {
			ships.clear();
		}
	}

}
