package uci.inf221.lunarLander.gui.impl;

import java.awt.AlphaComposite;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.GraphicsConfiguration;
import java.awt.GraphicsEnvironment;
import java.awt.Image;
import java.awt.Transparency;
import java.awt.geom.AffineTransform;
import java.awt.image.BufferedImage;

public class Sprite {
	private static final float TRANSLUCENT_ALPHA = 0.7f;
	private Image image;
	private Sprite translucentSprite;

	public Sprite(Image image) {
		this.image = image;
	}

	public int getWidth() {
		return image.getWidth(null);
	}

	public int getHeight() {
		return image.getHeight(null);
	}

	public void draw(Graphics g, float x, float y) {
		g.drawImage(image, Math.round(x - getWidth() / 2),
				Math.round(y - getHeight() / 2), null);
	}

	public void drawRotated(Graphics2D g, float angle, float x, float y) {
		AffineTransform at = new AffineTransform();
		at.translate(x, y);
		at.rotate(angle);
		at.translate(-getWidth() / 2, -getHeight() / 2);
		g.drawImage(image, at, null);
	}

	public void drawRotated(Graphics2D g, float angle, float x, float y,
			int xTranslate, int yTranslate) {
		AffineTransform at = new AffineTransform();
		at.translate(x, y);
		at.rotate(angle);
		at.translate(xTranslate, yTranslate);
		at.translate(-getWidth() / 2, -getHeight() / 2);
		g.drawImage(image, at, null);
	}

	public Sprite getTranslucentSprite() {
		if (translucentSprite == null) {
			GraphicsConfiguration gc = GraphicsEnvironment
					.getLocalGraphicsEnvironment().getDefaultScreenDevice()
					.getDefaultConfiguration();
			BufferedImage target = gc.createCompatibleImage(getWidth(),
					getHeight(), Transparency.TRANSLUCENT);
			Graphics2D g = target.createGraphics();

			g.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER,
					TRANSLUCENT_ALPHA));
			g.drawImage(image, 0, 0, null);
			g.dispose();

			translucentSprite = new Sprite(target);
		}
		return translucentSprite;
	}
}
