package uci.inf221.lunarLander.logic.component;

import java.util.UUID;

import uci.inf221.lunarLander.arch.Port.PortDirection;
import uci.inf221.lunarLander.arch.c2.C2Component;
import uci.inf221.lunarLander.arch.c2.C2Notification;
import uci.inf221.lunarLander.arch.c2.C2Request;
import uci.inf221.lunarLander.client.Client;
import uci.inf221.lunarLander.dataStore.entities.Level.LevelStatus;
import uci.inf221.lunarLander.dataStore.entities.MovementVector;
import uci.inf221.lunarLander.dataStore.entities.Ship;
import uci.inf221.lunarLander.dataStore.message.ClockTickNotification;
import uci.inf221.lunarLander.dataStore.message.EntityDestroyedNotification;
import uci.inf221.lunarLander.dataStore.message.LevelStatusUpdatedNotification;
import uci.inf221.lunarLander.dataStore.message.MovementVectorUpdatedNotification;
import uci.inf221.lunarLander.dataStore.message.NewShipNotification;
import uci.inf221.lunarLander.net.connector.serverToClients.entity.RemoteConnectionId;

public class ThresholdingLogicComponent extends C2Component {

	public static final String PORT_IDENTIFIER_DATA_STORE_CONNECTOR = "data_store";
	public static final String PORT_IDENTIFIER_CLIENT_TO_SERVER_CONNECTOR = "client_to_server";

	// TODO adjust constants ?
	private static final float POSITION_THRESHOLD = 1f; // pixels
	private static final long CLOCK_TICK_THRESHOLD = 1000; // milliseconds

	private Ship ship;

	@Override
	protected void createPorts() {
		addTopPort(PORT_IDENTIFIER_DATA_STORE_CONNECTOR).setDirection(
				PortDirection.IN);
		addBottomPort(PORT_IDENTIFIER_CLIENT_TO_SERVER_CONNECTOR).setDirection(
				PortDirection.OUT);
	}

	@Override
	protected void initialize() {
	}

	@Override
	public void start() {
	}

	@Override
	public void stop() {
	}

	@Override
	public void waitForStop() {
	}

	@Override
	protected void receiveNotification(C2Notification notification) {
		if (notification instanceof NewShipNotification) {
			Ship ship = ((NewShipNotification) notification).getShip();
			if (ship.getPlayerId().equals(Client.getPlayerId())) {
				synchronized (this) {
					this.ship = ship;
				}
			}
		} else if (notification instanceof EntityDestroyedNotification) {
			synchronized (this) {
				if (ship != null
						&& ship.getId().equals(
								((EntityDestroyedNotification) notification)
										.getEntityId())) {
					ship = null;
				}
			}
		} else if (notification instanceof LevelStatusUpdatedNotification) {
			LevelStatus levelStatus = ((LevelStatusUpdatedNotification) notification)
					.getLevelStatus();
			if (levelStatus.hasEnded()) {
				synchronized (this) {
					ship = null;
				}
			}
		}

		if (notification.getInitiatorData() == null
				|| !(notification.getInitiatorData() instanceof RemoteConnectionId)) {
			if (notification instanceof MovementVectorUpdatedNotification) {
				UUID entityId = ((MovementVectorUpdatedNotification) notification)
						.getEntityId();
				boolean send = false;
				synchronized (this) {
					if (ship != null && entityId.equals(ship.getId())) {
						MovementVector actualVector = ((MovementVectorUpdatedNotification) notification)
								.getMovementVector();

						if (actualVector.getClockTick()
								- ship.getMovementVector().getClockTick() > CLOCK_TICK_THRESHOLD) {
							send = true;
							ship.setMovementVector(actualVector);
						} else {
							MovementVector predictedVector = ship
									.getMovementVector().moveToTime(
											actualVector.getClockTick());

							if (vectorDifferenceBreachesThresholds(
									actualVector, predictedVector)) {
								send = true;
								ship.setMovementVector(actualVector);
							}
						}
					}
				}
				if (send) {
					sendNotification(notification);
				}
			} else if (!(notification instanceof ClockTickNotification)) {
				sendNotification(notification);
			}
		}
	}

	private boolean vectorDifferenceBreachesThresholds(
			MovementVector actualVector, MovementVector predictedVector) {
		float xDiff = Math.abs(actualVector.getPosition()[0]
				- predictedVector.getPosition()[0]);
		float yDiff = Math.abs(actualVector.getPosition()[1]
				- predictedVector.getPosition()[1]);

		if (xDiff > POSITION_THRESHOLD || yDiff > POSITION_THRESHOLD) {
			return true;
		}
		return false;
	}

	@Override
	protected void receiveRequest(C2Request request) {
	}

}
