package uci.inf221.lunarLander.managing.component;

import java.util.concurrent.atomic.AtomicBoolean;
import java.util.concurrent.atomic.AtomicLong;

import uci.inf221.lunarLander.arch.c2.C2Component;
import uci.inf221.lunarLander.arch.c2.C2Notification;
import uci.inf221.lunarLander.arch.c2.C2Request;
import uci.inf221.lunarLander.dataStore.entities.Asteroid;
import uci.inf221.lunarLander.dataStore.entities.Level.LevelStatus;
import uci.inf221.lunarLander.dataStore.entities.MovementVector;
import uci.inf221.lunarLander.dataStore.message.ClockTickNotification;
import uci.inf221.lunarLander.dataStore.message.CreateNewAsteroidRequest;
import uci.inf221.lunarLander.dataStore.message.LevelStatusUpdatedNotification;
import uci.inf221.lunarLander.dataStore.message.LevelUpdatedNotification;

public class AsteroidManagerComponent extends C2Component {

	public static final String PORT_IDENTIFIER_DATA_STORE_CONNECTOR = "data_store";

	// TODO adjust constants ?

	private static final long ASTEROID_MIN_INTERVAL = 500;
	private static final long ASTEROID_MAX_INTERVAL = 2000;
	private static final float ASTEROID_MIN_SPEED = 10f / 1000;
	private static final float ASTEROID_MAX_SPEED = 50f / 1000;

	private AtomicBoolean started = new AtomicBoolean(false);
	private AtomicLong lastAsteroidCreated = new AtomicLong(0);
	private AtomicLong nextAsteroidInterval = new AtomicLong(
			ASTEROID_MAX_INTERVAL);

	private int levelWidth;

	private int levelHeight;

	@Override
	protected void createPorts() {
		addTopPort(PORT_IDENTIFIER_DATA_STORE_CONNECTOR);
	}

	@Override
	protected void initialize() {
	}

	@Override
	public void start() {
	}

	@Override
	public void stop() {
	}

	@Override
	public void waitForStop() {
	}

	@Override
	protected void receiveNotification(C2Notification notification) {
		if (notification instanceof LevelStatusUpdatedNotification) {
			LevelStatus status = ((LevelStatusUpdatedNotification) notification)
					.getLevelStatus();
			if (status.hasEnded()) {
				started.set(false);
			} else if (status == LevelStatus.STARTED) {
				lastAsteroidCreated.set(0);
				nextAsteroidInterval.set(ASTEROID_MAX_INTERVAL);
				started.set(true);
			}
		} else if (notification instanceof LevelUpdatedNotification) {
			levelWidth = ((LevelUpdatedNotification) notification).getLevel()
					.getWidth();
			levelHeight = ((LevelUpdatedNotification) notification).getLevel()
					.getHeight();
		} else if (notification instanceof ClockTickNotification) {
			if (started.get()) {
				if (((ClockTickNotification) notification).getTickTime() > this.lastAsteroidCreated
						.get() + nextAsteroidInterval.get()) {
					createNewAsteroid();
					this.lastAsteroidCreated
							.set(((ClockTickNotification) notification)
									.getTickTime());
				}
			}
		}
	}

	private void createNewAsteroid() {
		Asteroid asteroid = new Asteroid();
		MovementVector movementVector = new MovementVector();

		boolean movingRight = Math.random() >= 0.5;

		float[] position = new float[] { movingRight ? 0 : levelWidth - 1,
				(float) (levelHeight / 4 * 3 * Math.random()) };
		movementVector.setPosition(position);

		float[] speed = new float[] {
				(float) (movingRight ? 0 : Math.PI),
				(float) ((ASTEROID_MAX_SPEED - ASTEROID_MIN_SPEED)
						* Math.random() + ASTEROID_MIN_SPEED) };
		movementVector.setSpeed(speed);

		asteroid.setMovementVector(movementVector);

		long interval = (long) Math
				.ceil((ASTEROID_MAX_INTERVAL - ASTEROID_MIN_INTERVAL)
						* Math.random() + ASTEROID_MIN_INTERVAL);
		nextAsteroidInterval.set(interval);

		sendRequest(new CreateNewAsteroidRequest(this, asteroid));
	}

	@Override
	protected void receiveRequest(C2Request request) {
	}

}
