package uci.inf221.lunarLander.net.connector;

import uci.inf221.lunarLander.arch.c2.C2Connector;
import uci.inf221.lunarLander.net.connector.clientToServer.component.ClientMessageComposerComponent;
import uci.inf221.lunarLander.net.connector.clientToServer.component.ServerConnectionManagerComponent;
import uci.inf221.lunarLander.net.connector.clientToServer.component.ServerConnectionStateChangeHandlerComponent;
import uci.inf221.lunarLander.net.connector.clientToServer.component.ServerMessageHandlerComponent;
import uci.inf221.lunarLander.net.connector.clientToServer.connector.ConnectionManagerConnector;
import uci.inf221.lunarLander.net.connector.clientToServer.connector.DataStorePortConnector;
import uci.inf221.lunarLander.net.connector.clientToServer.connector.ThresholdingPortConnector;

public class ClientToServerConnector extends C2Connector {

	public static final String PORT_IDENTIFIER_DATA_STORE_CONNECTOR = "data_store";
	public static final String PORT_IDENTIFIER_THRESHOLDING_LOGIC_COMPONENT = "thresholding_logic";

	private ClientMessageComposerComponent clientMessageComposerComponent;
	private ServerMessageHandlerComponent serverMessageHandlerComponent;
	private ServerConnectionStateChangeHandlerComponent serverConnectionStateChangeHandlerComponent;
	private ServerConnectionManagerComponent serverConnectionManagerComponent;

	private ThresholdingPortConnector thresholdingPortConnector;
	private DataStorePortConnector dataStorePortConnector;
	private ConnectionManagerConnector connectionManagerConnector;

	@Override
	protected void createPorts() {
		createInnerStructure();

		addPort(PORT_IDENTIFIER_DATA_STORE_CONNECTOR,
				dataStorePortConnector
						.getPort(DataStorePortConnector.PORT_IDENTIFIER_CLIENT_TO_SERVER_CONNECTOR));
		addPort(PORT_IDENTIFIER_THRESHOLDING_LOGIC_COMPONENT,
				thresholdingPortConnector
						.getPort(ThresholdingPortConnector.PORT_IDENTIFIER_CLIENT_TO_SERVER_CONNECTOR));
	}

	private void createInnerStructure() {
		clientMessageComposerComponent = new ClientMessageComposerComponent();
		serverMessageHandlerComponent = new ServerMessageHandlerComponent();
		serverConnectionStateChangeHandlerComponent = new ServerConnectionStateChangeHandlerComponent();
		serverConnectionManagerComponent = new ServerConnectionManagerComponent();

		thresholdingPortConnector = new ThresholdingPortConnector();
		dataStorePortConnector = new DataStorePortConnector();
		connectionManagerConnector = new ConnectionManagerConnector();

		thresholdingPortConnector
				.getPort(
						ThresholdingPortConnector.PORT_IDENTIFIER_CLIENT_MESSAGE_COMPOSER_COMPONENT)
				.connectTo(
						clientMessageComposerComponent
								.getPort(ClientMessageComposerComponent.PORT_IDENTIFIER_THRESHOLDING_PORT_CONNECTOR));

		dataStorePortConnector
				.getPort(
						DataStorePortConnector.PORT_IDENTIFIER_SERVER_MESSAGE_HANDLER_COMPONENT)
				.connectTo(
						serverMessageHandlerComponent
								.getPort(ServerMessageHandlerComponent.PORT_IDENTIFIER_DATA_STORE_PORT_CONNECTOR));
		dataStorePortConnector
				.getPort(
						DataStorePortConnector.PORT_IDENTIFIER_SERVER_CONNECTION_STATE_CHANGE_HANDLER_COMPONENT)
				.connectTo(
						serverConnectionStateChangeHandlerComponent
								.getPort(ServerConnectionStateChangeHandlerComponent.PORT_IDENTIFIER_DATA_STORE_PORT_CONNECTOR));

		connectionManagerConnector
				.getPort(
						ConnectionManagerConnector.PORT_IDENTIFIER_CLIENT_MESSAGE_COMPOSER_COMPONENT)
				.connectTo(
						clientMessageComposerComponent
								.getPort(ClientMessageComposerComponent.PORT_IDENTIFIER_CONNECTION_MANAGER_CONNECTOR));
		connectionManagerConnector
				.getPort(
						ConnectionManagerConnector.PORT_IDENTIFIER_SERVER_MESSAGE_HANDLER_COMPONENT)
				.connectTo(
						serverMessageHandlerComponent
								.getPort(ServerMessageHandlerComponent.PORT_IDENTIFIER_CONNECTION_MANAGER_CONNECTOR));
		connectionManagerConnector
				.getPort(
						ConnectionManagerConnector.PORT_IDENTIFIER_SERVER_CONNECTION_STATE_CHANGE_HANDLER_COMPONENT)
				.connectTo(
						serverConnectionStateChangeHandlerComponent
								.getPort(ServerConnectionStateChangeHandlerComponent.PORT_IDENTIFIER_CONNECTION_MANAGER_CONNECTOR));
		connectionManagerConnector
				.getPort(
						ConnectionManagerConnector.PORT_IDENTIFIER_SERVER_CONNECTION_MANAGER_COMPONENT)
				.connectTo(
						serverConnectionManagerComponent
								.getPort(ServerConnectionManagerComponent.PORT_IDENTIFIER_CONNECTION_MANAGER_CONNECTOR));
	}

	@Override
	public void start() {
		clientMessageComposerComponent.start();
		serverMessageHandlerComponent.start();
		serverConnectionStateChangeHandlerComponent.start();
		serverConnectionManagerComponent.start();

		dataStorePortConnector.start();
		thresholdingPortConnector.start();
		connectionManagerConnector.start();
	}

	@Override
	public void stop() {
		serverConnectionManagerComponent.stop();
		serverConnectionStateChangeHandlerComponent.stop();
		serverMessageHandlerComponent.stop();
		clientMessageComposerComponent.stop();

		connectionManagerConnector.stop();
		thresholdingPortConnector.stop();
		dataStorePortConnector.stop();
	}

	public void waitForStop() {
		serverConnectionManagerComponent.waitForStop();
		serverConnectionStateChangeHandlerComponent.waitForStop();
		serverMessageHandlerComponent.waitForStop();
		clientMessageComposerComponent.waitForStop();

		connectionManagerConnector.waitForStop();
		thresholdingPortConnector.waitForStop();
		dataStorePortConnector.waitForStop();
	}

}
