package uci.inf221.lunarLander.net.connector.serverToClients.component;

import uci.inf221.lunarLander.arch.Port.PortDirection;
import uci.inf221.lunarLander.arch.c2.C2Component;
import uci.inf221.lunarLander.arch.c2.C2Notification;
import uci.inf221.lunarLander.arch.c2.C2Request;
import uci.inf221.lunarLander.dataStore.message.ClockTickNotification;
import uci.inf221.lunarLander.dataStore.message.CurrentEntityDataNotification;
import uci.inf221.lunarLander.dataStore.message.CurrentGameStartTimeNotification;
import uci.inf221.lunarLander.dataStore.message.MovementVectorUpdatedNotification;
import uci.inf221.lunarLander.net.connector.serverToClients.entity.RemoteConnectionId;
import uci.inf221.lunarLander.net.connector.serverToClients.message.ClientStateUpdatedNotification;
import uci.inf221.lunarLander.net.connector.serverToClients.message.NewMessageNotification;
import uci.inf221.lunarLander.net.connector.serverToClients.message.NewMessageNotification.TargetMode;

public class ServerMessageComposerComponent extends C2Component {

	public static final String PORT_IDENTIFIER_DATA_STORE_PORT_CONNECTOR = "data_store_port";
	public static final String PORT_IDENTIFIER_CONNECTION_MANAGER_CONNECTOR = "connection_manager";

	@Override
	protected void createPorts() {
		addTopPort(PORT_IDENTIFIER_DATA_STORE_PORT_CONNECTOR).setDirection(
				PortDirection.IN);
		addBottomPort(PORT_IDENTIFIER_CONNECTION_MANAGER_CONNECTOR)
				.setDirection(PortDirection.OUT);
	}

	@Override
	protected void initialize() {
	}

	@Override
	public void start() {
	}

	@Override
	public void stop() {
	}

	@Override
	public void waitForStop() {
	}

	@Override
	protected void receiveNotification(C2Notification notification) {
		if (!(notification instanceof ClientStateUpdatedNotification)
				&& !(notification instanceof ClockTickNotification)) {
			boolean important = !(notification instanceof MovementVectorUpdatedNotification);
			if (notification.getInitiatorData() instanceof RemoteConnectionId) {
				int remoteId = ((RemoteConnectionId) notification
						.getInitiatorData()).getRemoteConnectionId();
				if (notification instanceof CurrentEntityDataNotification
						|| notification instanceof CurrentGameStartTimeNotification) {
					sendNotification(new NewMessageNotification(this,
							notification, important, remoteId,
							TargetMode.SINGLE));
				} else {
					sendNotification(new NewMessageNotification(this,
							notification, important, remoteId,
							TargetMode.EXCLUDING));
				}
			} else {
				sendNotification(new NewMessageNotification(this, notification,
						important));
			}
		}
	}

	@Override
	protected void receiveRequest(C2Request request) {
	}

}
