package uci.inf221.lunarLander.utils;

import java.util.UUID;

import uci.inf221.lunarLander.dataStore.entities.Asteroid;
import uci.inf221.lunarLander.dataStore.entities.Bullet;
import uci.inf221.lunarLander.dataStore.entities.DataEntity;
import uci.inf221.lunarLander.dataStore.entities.Level;
import uci.inf221.lunarLander.dataStore.entities.Level.LevelStatus;
import uci.inf221.lunarLander.dataStore.entities.MovementVector;
import uci.inf221.lunarLander.dataStore.entities.Ship;
import uci.inf221.lunarLander.dataStore.entities.Status;
import uci.inf221.lunarLander.dataStore.message.CurrentEntityDataNotification;
import uci.inf221.lunarLander.dataStore.message.CurrentGameStartTimeNotification;
import uci.inf221.lunarLander.dataStore.message.EntityDestroyedNotification;
import uci.inf221.lunarLander.dataStore.message.GameStartTimeUpdatedNotification;
import uci.inf221.lunarLander.dataStore.message.LevelStatusUpdatedNotification;
import uci.inf221.lunarLander.dataStore.message.LevelUpdatedNotification;
import uci.inf221.lunarLander.dataStore.message.MovementVectorUpdatedNotification;
import uci.inf221.lunarLander.dataStore.message.NewAsteroidNotification;
import uci.inf221.lunarLander.dataStore.message.NewBulletNotification;
import uci.inf221.lunarLander.dataStore.message.NewShipNotification;
import uci.inf221.lunarLander.net.connector.serverToClients.entity.ClientState;

import com.esotericsoftware.kryo.Kryo;
import com.esotericsoftware.kryo.Serializer;
import com.esotericsoftware.kryo.io.Input;
import com.esotericsoftware.kryo.io.Output;

public class KryoInitializer {

	public static class UUIDSerializer extends Serializer<UUID> {
		@Override
		public void write(Kryo kryo, Output output, UUID uuid) {
			output.writeLong(uuid.getMostSignificantBits(), false);
			output.writeLong(uuid.getLeastSignificantBits(), false);
		}

		@Override
		public UUID read(Kryo kryo, Input input, Class<UUID> uuidClass) {
			return new UUID(input.readLong(false), input.readLong(false));
		}
	}

	public static void initKryo(Kryo kryo) {
		kryo.register(UUID.class);
		kryo.register(DataEntity.class);
		kryo.register(Asteroid.class);
		kryo.register(Bullet.class);
		kryo.register(Level.class);
		kryo.register(LevelStatus.class);
		kryo.register(Ship.class);
		kryo.register(MovementVector.class);
		kryo.register(Status.class);
		kryo.register(ClientState.class);
		kryo.register(EntityDestroyedNotification.class);
		kryo.register(LevelStatusUpdatedNotification.class);
		kryo.register(LevelUpdatedNotification.class);
		kryo.register(MovementVectorUpdatedNotification.class);
		kryo.register(NewAsteroidNotification.class);
		kryo.register(NewBulletNotification.class);
		kryo.register(NewShipNotification.class);
		kryo.register(CurrentGameStartTimeNotification.class);
		kryo.register(CurrentEntityDataNotification.class);
		kryo.register(GameStartTimeUpdatedNotification.class);
		kryo.register(UUID.class, new UUIDSerializer());

		kryo.register(DataEntity[].class);
		kryo.register(float[].class);
	}

}
