package uci.inf221.lunarLander.utils;

import java.util.concurrent.atomic.AtomicBoolean;

public class TerminateableLoopThread extends Thread {

	private static final int SLEEP_TIME = 10;

	private Runnable loopBody;
	private AtomicBoolean terminated = new AtomicBoolean(false);
	private int sleepTime;

	public TerminateableLoopThread(Runnable loopBody) {
		this(loopBody, SLEEP_TIME);
	}

	public TerminateableLoopThread(Runnable loopBody, int sleepTime) {
		super();
		this.loopBody = loopBody;
		this.sleepTime = sleepTime;
	}

	@Override
	public void run() {
		while (!terminated.get()) {
			loopBody.run();
			try {
				Thread.sleep(sleepTime);
			} catch (InterruptedException e) {
			}
		}
	}

	public void terminate() {
		terminated.set(true);
	}

	public void waitForTermination() {
		while (isAlive()) {
			try {
				Thread.sleep(1);
			} catch (InterruptedException e) {
			}
		}
	}

	public boolean isTerminated() {
		return terminated.get();
	}
}
