package org.cynetec.lib.system.net.ntp;

import java.io.IOException;
import java.net.DatagramPacket;
import java.net.DatagramSocket;
import java.util.concurrent.atomic.AtomicBoolean;

import org.apache.commons.net.ntp.NtpV3Impl;
import org.apache.commons.net.ntp.NtpV3Packet;
import org.apache.commons.net.ntp.TimeStamp;
import org.apache.log4j.Logger;

public class NTPUDPServer extends Thread {
	private Logger log = Logger.getLogger(this.getClass());

	/*** The default NTP port. It is set to 123 according to RFC 1305. ***/
	public static final int DEFAULT_PORT = 123;

	private AtomicBoolean terminated;
	private DatagramSocket datagramSocket;

	public NTPUDPServer() throws IOException {
		this(DEFAULT_PORT);
	}

	public NTPUDPServer(int port) throws IOException {
		terminated = new AtomicBoolean(false);

		// create datagram socket
		datagramSocket = new DatagramSocket(port);
	}

	@Override
	public void run() {
		NtpV3Packet recMessage = new NtpV3Impl();
		DatagramPacket receivePacket = recMessage.getDatagramPacket();

		while (!terminated.get() && !datagramSocket.isClosed()) {
			try {
				datagramSocket.receive(receivePacket);
				handlePacket(recMessage, System.currentTimeMillis());

				recMessage = new NtpV3Impl();
				receivePacket = recMessage.getDatagramPacket();
			} catch (IOException e) {
				log.warn("could not receive datagram packet", e);
			}

			try {
				Thread.sleep(25);
			} catch (InterruptedException e) {
			}
		}
	}

	private void handlePacket(NtpV3Packet recMessage, long receiveTimeMillis) {
		NtpV3Packet message = new NtpV3Impl();
		DatagramPacket sendPacket = message.getDatagramPacket();
		sendPacket.setAddress(recMessage.getDatagramPacket().getAddress());
		sendPacket.setPort(recMessage.getDatagramPacket().getPort());

		message.setMode(NtpV3Packet.MODE_SERVER);
		message.setVersion(NtpV3Packet.VERSION_3);
		message.setLeapIndicator(NtpV3Packet.LI_NO_WARNING);
		message.setStratum(1);
		message.setPoll(NtpV3Packet.NTP_MINPOLL);
		message.setReferenceId(NtpV3Packet.NTP_MINCLOCK);

		message.setOriginateTimeStamp(recMessage.getTransmitTimeStamp());
		message.setReceiveTimeStamp(TimeStamp.getNtpTime(receiveTimeMillis));

		/*
		 * Must minimize the time between getting the current time, timestamping
		 * the packet, and sending it out which introduces an error in the delay
		 * time. No extraneous logging and initializations here !!!
		 */
		TimeStamp now = TimeStamp.getCurrentTime();
		message.setReferenceTime(now);
		message.setTransmitTime(now);

		try {
			datagramSocket.send(sendPacket);
		} catch (IOException e) {
			log.warn("could not send response datagram packet", e);
		}
	}

	public void terminate() {
		terminated.set(true);
		this.interrupt();
	}

}
